@@ -5,12 +5,11 @@
 
   @author Johan Hedberg <johan.hedberg@nokia.com>
 
-  Copyright (C) 2004 Nokia. All rights reserved.
+  Copyright (C) 2004-2006 Nokia Corporation. All rights reserved.
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2 of the License, or (at your option) any later version.
+  License, version 2.1, as published by the Free Software Foundation.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -155,8 +154,6 @@ static const char *optostr(uint8_t op) {
             return "Put";
         case OBEX_CMD_GET:
             return "Get";
-        case OBEX_CMD_COMMAND:
-            return "Command";
         case OBEX_CMD_SETPATH:
             return "SetPath";
         case OBEX_CMD_ABORT:
@@ -248,8 +245,9 @@ static void show_headers(obex_t *handle, obex_object_t *object) {
                 g_free(str);
                 break;
             case OBEX_HDR_TIME:
-                str = g_strdup_printf("OBEX_HDR_TIME: %%%us\n", hlen);
-                debug(str, hv.bs);
+		str = g_new0(char, hlen + 1);
+		memcpy(str, hv.bs, hlen);
+                debug("OBEX_HDR_TIME: %s\n", str);
                 g_free(str);
                 break;
             case OBEX_HDR_TYPE:
@@ -382,8 +380,21 @@ static void obex_readstream(GwObex *ctx, obex_object_t *object) {
 
             xfer->do_cb = TRUE;
         }
-        else if (xfer->stream_fd >= 0)
-            (void) write(xfer->stream_fd, buf, actual);
+        else if (xfer->stream_fd >= 0) {
+            int written = 0;
+
+            while (written < actual) {
+                int ret;
+
+                ret = write(xfer->stream_fd, buf, actual);
+                if (ret < 0 && errno != EINTR) {
+                    debug("Could not write: %s (%d)", g_strerror(errno), errno);
+                    break;
+                }
+
+                written += ret;
+            }
+        }
         else {
             xfer->buf = g_realloc(xfer->buf, xfer->counter);
             memcpy(&xfer->buf[xfer->buf_size], buf, actual);
@@ -632,7 +643,9 @@ gboolean gw_obex_cb(GIOChannel *chan, GIOCondition cond, gpointer data) {
 gboolean gw_obex_disconnect(GwObex *ctx) {
     obex_object_t *object;
 
-    if (!ctx->done || ctx->xfer) {
+    g_assert(!ctx->xfer);
+
+    if (!ctx->done) {
         ctx->error = GW_OBEX_ERROR_BUSY;
         return FALSE;
     }
@@ -652,10 +665,7 @@ gboolean gw_obex_connect(GwObex *ctx, const char *target, size_t target_len) {
     gboolean ret;
     obex_object_t *object;
 
-    if (!ctx->done || ctx->xfer) {
-        ctx->error = GW_OBEX_ERROR_BUSY;
-        return FALSE;
-    }
+    g_assert(ctx->done && !ctx->xfer);
 
     ctx->obex_op = OBEX_CMD_CONNECT;
 
@@ -814,7 +824,7 @@ gboolean gw_obex_get(GwObex *ctx,
 
     ctx->obex_op = OBEX_CMD_GET;
 
-    ctx->xfer = gw_obex_xfer_new(ctx, async);
+    ctx->xfer = gw_obex_xfer_new(ctx, async, stream_fd);
 
     object = OBEX_ObjectNew(ctx->handle, OBEX_CMD_GET);
 
@@ -934,7 +944,7 @@ gboolean gw_obex_put(GwObex *ctx,
         }
     }
 
-    ctx->xfer = gw_obex_xfer_new(ctx, async);
+    ctx->xfer = gw_obex_xfer_new(ctx, async, stream_fd);
 
     if (local) {
         if (file_is_dir(local)) {
@@ -961,9 +971,7 @@ gboolean gw_obex_put(GwObex *ctx,
         ctx->xfer->buf_size = object_size;
         debug("Sending to %s\n", remote ? remote : type);
     }
-    else if (stream_fd >= 0)
-        ctx->xfer->stream_fd = stream_fd;
-    else if (!async) { /* Delete */
+    else if (stream_fd < 0 && !async) { /* Delete */
         ctx->report_progress = FALSE;
         debug("Deleting %s\n", remote ? remote : type);
     }
@@ -1042,6 +1050,9 @@ out:
     if (ctx->xfer->stream_fd >= 0 && stream_fd < 0)
         close(ctx->xfer->stream_fd);
 
+    if (buf)
+        ctx->xfer->buf = NULL;
+
     _gw_obex_xfer_free(ctx->xfer);
     ctx->xfer = NULL;
 
